@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -110,7 +110,7 @@ def getAuthInfo(id, reason, defaultLogin=None, force=False, parent=None):
         mTxtLogin  = wTree.get_widget('txt-login')
         mTxtPasswd = wTree.get_widget('txt-passwd')
 
-        wTree.get_widget('lbl-reason').set_text('%s\n%s.' % (_('Enter your username and password for'), reason))
+        wTree.get_widget('lbl-reason').set_text(_('Enter your username and password for\n%(reason)s') % {'reason': reason})
         wTree.get_widget('dlg-action_area').set_child_secondary(wTree.get_widget('btn-help'), True)   # Glade fails to do that
         wTree.get_widget('lbl-title').set_markup('<big><big><big><b>%s</b></big></big></big>' % _('Password required'))
 
