@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 #
-# ExtListView v1.4
+# ExtListView v1.5
+#
+# v1.5:
+#   * Fixed intermittent improper columns resizing
 #
 # v1.4:
 #   * Replaced TYPE_INT by TYPE_PYOBJECT as the fifth parameter type of extListview-dnd
@@ -337,6 +340,8 @@ class ExtListView(gtk.TreeView):
         self.__resetSorting()
         self.clearMark()
         self.store.clear()
+        # This fixes the problem of columns sometimes not resizing correctly
+        self.resize_children()
 
 
     def setItem(self, rowIndex, colIndex, value):
@@ -363,6 +368,8 @@ class ExtListView(gtk.TreeView):
         if len(self.store) == 0:
             self.set_cursor(0)
             self.__resetSorting()
+        # This fixes the problem of columns sometimes not resizing correctly
+        self.resize_children()
         self.emit('extlistview-modified')
 
 
@@ -415,7 +422,7 @@ class ExtListView(gtk.TreeView):
         self.freeze_child_notify()
         self.set_model(None)
         self.clear()
-        self.insertRows(rows)
+        self.appendRows(rows)
         self.set_model(self.store)
         self.thaw_child_notify()
 
