@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,16 +23,16 @@ import media, os.path
 mFormats = {'.m3u': None}
 
 
-def getSupportedFormats():
-    """ Return a list of supported playlist formats """
-    return ['*.m3u']
-
-
 def isSupported(file):
     """ Return True if the file has a supported format """
     return file.lower()[-4:] in mFormats
 
 
+def getSupportedFormats():
+    """ Return a list of supported playlist formats """
+    return ['*' + key for key in mFormats]
+
+
 def save(files, playlist):
     """ Create a playlist with the given files """
     output = open(playlist, 'w')
@@ -46,7 +46,7 @@ def load(playlist):
         return []
 
     input = open(playlist)
-    files = [strippedLine for strippedLine in [line.strip() for line in input.readlines()] if len(strippedLine) != 0 and strippedLine[0] != '#']
+    files = [line for line in [line.strip() for line in input] if len(line) != 0 and line[0] != '#']
     input.close()
 
     path = os.path.dirname(playlist)
