@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,10 +18,11 @@
 
 import cgi, gobject, gtk, gui, modules, os, socket, tools, traceback, urllib
 
-from gui       import extTreeview
-from tools     import consts, prefs, sec2str
-from gettext   import gettext as _
-from tools.log import logger
+from gui                 import extTreeview
+from tools               import consts, prefs, sec2str
+from gettext             import gettext as _
+from tools.log           import logger
+from media.track.cdTrack import CDTrack
 
 MOD_INFO = ('Audio CD', _('Audio CD'), _('Play audio discs'), ('DiscID', 'CDDB'), False, True)
 
@@ -96,8 +97,8 @@ class AudioCD(modules.ThreadedModule):
         """ Replace/extend the tracklist """
         tracks = self.getTracksFromSelectedRows(tree)
 
-        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET_EXT, {'files': tracks, 'playNow': True})
-        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD_EXT, {'files': tracks})
+        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
+        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
     # --== Cache management ==--
@@ -138,7 +139,7 @@ class AudioCD(modules.ThreadedModule):
         self.tree.replaceContent(((consts.icoCdrom, name, None),))
 
         # Append a child for each track
-        self.tree.appendRows([(consts.icoMediaFile, _('Track') + (' %02u' % (i+1)), None) for i in xrange(nbTracks)], (0,))
+        self.tree.appendRows([(consts.icoMediaFile, _('Track %02u') % (i+1), None) for i in xrange(nbTracks)], (0,))
         self.tree.expand_all()
 
 
@@ -159,14 +160,33 @@ class AudioCD(modules.ThreadedModule):
         # Update the root of the tree
         disc          = cddb['DTITLE'].strip().decode('iso-8859-15', 'replace')
         rowName       = '<span size="smaller" foreground="#909090">[%s]</span>  %s' % (sec2str(sum(trackLen)), cgi.escape(disc))
-        artist, album = disc.split('/')
+        artist, album = disc.split(' / ')
         self.tree.setItem((0,), ROW_NAME, rowName)
 
+        # Optional information
+        try:    date = cddb['DYEAR'].strip().decode('iso-8859-15', 'replace')
+        except: date = None
+
+        try:    genre = cddb['DGENRE'].strip().decode('iso-8859-15', 'replace')
+        except: genre = None
+
         # Update each track
         for i, child in enumerate(self.tree.iterChildren((0,))):
             title = cddb['TTITLE%u' % i].strip().decode('iso-8859-15', 'replace')
+
+            # Create the corresponding Track object
+            track = CDTrack(str(i+1))
+            track.setTitle(title)
+            track.setAlbum(album)
+            track.setArtist(artist)
+            track.setLength(trackLen[i])
+            track.setNumber(i+1)
+            # Optional information
+            if date is not None:  track.setDate(date)
+            if genre is not None: track.setGenre(genre)
+            # Fill the tree
             self.tree.setItem(child, ROW_NAME, '%02u. %s' % (i + 1, cgi.escape(title)))
-            self.tree.setItem(child, ROW_TRACK, [i + 1, title, artist, album, trackLen[i], 'cdda://%u' % (i + 1)])
+            self.tree.setItem(child, ROW_TRACK, track)
 
 
     # --== Disc management ==--
@@ -242,10 +262,8 @@ class AudioCD(modules.ThreadedModule):
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        tracks    = self.getTracksFromSelectedRows(tree)
-        allFields = '\n'.join([' '.join([urllib.pathname2url(str(field)) for field in track]) for track in tracks])
-
-        selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, allFields)
+        serializedTracks = '\n'.join([track.serialize() for track in self.getTracksFromSelectedRows(tree)])
+        selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, serializedTracks)
 
 
     def onShowPopupMenu(self, tree, button, time, path):
@@ -270,8 +288,15 @@ class AudioCD(modules.ThreadedModule):
 
     def onKeyPressed(self, tree, event):
         """ A key has been pressed """
-        if gtk.gdk.keyval_name(event.keyval) == 'F5':
-            self.reloadDisc()
+        keyname = gtk.gdk.keyval_name(event.keyval)
+
+        if keyname == 'F5':       self.reloadDisc()
+        elif keyname == 'plus':   tree.expandRows()
+        elif keyname == 'Left':   tree.collapseRows()
+        elif keyname == 'Right':  tree.expandRows()
+        elif keyname == 'minus':  tree.collapseRows()
+        elif keyname == 'space':  tree.switchRows()
+        elif keyname == 'Return': self.playSelection(tree, True)
 
 
     # --== Configuration ==--
