@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import md5, modules, os.path, socket, traceback
+import md5, modules, os.path, socket, tools, traceback
 
 from gui       import authentication
 from time      import time, sleep
 from tools     import consts
-from urllib    import quote_plus
 from urllib2   import urlopen
 from gettext   import gettext as _
 from tools.log import logger
@@ -29,7 +28,7 @@ from tools.log import logger
 MOD_INFO = ('AudioScrobbler', 'AudioScrobbler', _('Keep your Last.fm profile up to date'), [], False, False)
 
 CLI_ID         = 'dbl'
-CLI_VER        = '0.1'
+CLI_VER        = '0.2'
 MOD_NAME       = MOD_INFO[modules.MODINFO_NAME]
 PROTO_VER      = '1.2'
 AS_SERVER      = 'post.audioscrobbler.com'
@@ -102,19 +101,19 @@ class AudioScrobbler(modules.ThreadedModule):
         if not (track.hasArtist() and track.hasTitle() and track.hasLength()):
             return
 
-        if not(track.getLengthSec() >= 30 and (self.currTrack[TRK_PLAY_TIME] >= 240 or self.currTrack[TRK_PLAY_TIME] >= track.getLengthSec()/2)):
+        if not(track.getLength() >= 30 and (self.currTrack[TRK_PLAY_TIME] >= 240 or self.currTrack[TRK_PLAY_TIME] >= track.getLength()/2)):
             return
 
         params = (
-                    ( 'a[*]', quote_plus(track.getSafeArtist())          ),
-                    ( 't[*]', quote_plus(track.getSafeTitle())           ),
+                    ( 'a[*]', tools.percentEncode(track.getSafeArtist()) ),
+                    ( 't[*]', tools.percentEncode(track.getSafeTitle())  ),
                     ( 'i[*]', str(self.currTrack[TRK_STARTED_TIMESTAMP]) ),
                     ( 'o[*]', 'P'                                        ),
                     ( 'r[*]', ''                                         ),
                     ( 'l[*]', track.getSafeLength()                      ),
-                    ( 'b[*]', quote_plus(track.getSafeAlbum())           ),
+                    ( 'b[*]', tools.percentEncode(track.getSafeAlbum())  ),
                     ( 'n[*]', track.getSafeNumber()                      ),
-                    ( 'm[*]', ''                                         )
+                    ( 'm[*]', track.getSafeMBTrackId()                   )
                  )
 
         self.cache.append('&'.join(['%s=%s' % (key, val) for (key, val) in params]))
@@ -212,13 +211,13 @@ class AudioScrobbler(modules.ThreadedModule):
             return
 
         params = (
-                    ( 's', self.session[SESSION_ID]          ),
-                    ( 'a', quote_plus(track.getSafeArtist()) ),
-                    ( 't', quote_plus(track.getSafeTitle())  ),
-                    ( 'b', quote_plus(track.getSafeAlbum())  ),
-                    ( 'l', track.getSafeLength()             ),
-                    ( 'n', track.getSafeNumber()             ),
-                    ( 'm', ''                                )
+                    ( 's', self.session[SESSION_ID]                   ),
+                    ( 'a', tools.percentEncode(track.getSafeArtist()) ),
+                    ( 't', tools.percentEncode(track.getSafeTitle())  ),
+                    ( 'b', tools.percentEncode(track.getSafeAlbum())  ),
+                    ( 'l', track.getSafeLength()                      ),
+                    ( 'n', track.getSafeNumber()                      ),
+                    ( 'm', track.getSafeMBTrackId()                   )
                  )
 
         try:
