@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -67,8 +67,10 @@ class CtrlPanel(modules.Module):
         # We need to keep the handler ID for this one to be able to disconnect it when needed
         self.seekHandler   = self.sclSeek.connect('value-changed',   self.onSeekValueChanged)
         self.volumeHandler = self.btnVolume.connect('value-changed', self.onVolumeValueChanged)
-        # This must be done only when the handler has been connected
-        self.btnVolume.set_value(prefs.get(__name__, 'volume', PREFS_DEFAULT_VOLUME))
+        # We must make sure that the handler will be called: this is not the case if the new value is the same as the old one
+        volumeValue = prefs.get(__name__, 'volume', PREFS_DEFAULT_VOLUME)
+        if self.btnVolume.get_value() != volumeValue: self.btnVolume.set_value(volumeValue)
+        else:                                         self.onVolumeValueChanged(self.btnVolume, volumeValue)
 
 
     def onNewTrack(self, track):
@@ -77,7 +79,7 @@ class CtrlPanel(modules.Module):
         self.btnPlay.set_sensitive(True)
         self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
         gtk.tooltips_data_get(self.btnPlay)[0].set_tip(self.btnPlay, _('Pause the current track'))
-        self.currTrackLength = track.getLengthSec()
+        self.currTrackLength = track.getLength()
         self.sclSeek.show()
         self.lblElapsed.show()
         self.lblRemaining.show()
