@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ class FileExplorer(modules.Module):
                    (None, [(None, TYPE_INT)],                                                                  False),
                    (None, [(None, TYPE_STRING)],                                                               False))
 
-        self.tree            = extTreeview.ExtTreeView(columns)
+        self.tree            = extTreeview.ExtTreeView(columns, True)
         self.popup           = None
         self.cfgWin          = None
         self.folders         = prefs.get(__name__, 'media-folders',     PREFS_DEFAULT_MEDIA_FOLDERS)
@@ -73,7 +73,7 @@ class FileExplorer(modules.Module):
         self.currFolder      = None
         self.showHiddenFiles = prefs.get(__name__, 'show-hidden-files', PREFS_DEFAULT_SHOW_HIDDEN_FILES)
         # Explorer
-        self.tree.setIsDraggableFunc(self.isDraggable)
+        self.tree.setIsDraggableFunc(self.isPlayable)
         self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_URI]])
         self.scrolled.add(self.tree)
         self.scrolled.set_shadow_type(gtk.SHADOW_IN)
@@ -101,8 +101,8 @@ class FileExplorer(modules.Module):
         return row[ROW_NAME].lower()
 
 
-    def isDraggable(self):
-        """ Determine whether the selected rows can be dragged """
+    def isPlayable(self):
+        """ Only playable rows can be dragged """
         for row in self.tree.iterSelectedRows():
             if row[ROW_TYPE] in (TYPE_FILE, TYPE_PLIST) or (row[ROW_TYPE] == TYPE_DIR and row[ROW_PIXBUF] == consts.icoMediaDir):
                 return True
@@ -122,13 +122,15 @@ class FileExplorer(modules.Module):
 
     def getFilesFromSelection(self, tree):
         """ Return a list of playable files from the selected rows """
-        return ['file://' + row[ROW_FULLPATH] for row in tree.getSelectedRows() if row[ROW_TYPE] != TYPE_NONE]
+        return [row[ROW_FULLPATH] for row in tree.getSelectedRows() if row[ROW_TYPE] != TYPE_NONE]
 
 
     def playSelection(self, tree, replace):
         """ Replace/extend the tracklist """
-        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'files': self.getFilesFromSelection(tree), 'playNow': True})
-        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'files': self.getFilesFromSelection(tree)})
+        if self.isPlayable():
+            tracks = media.getTracks(self.getFilesFromSelection(tree))
+            if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
+            else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
     def renameFolder(self, oldName, newName):
@@ -143,7 +145,23 @@ class FileExplorer(modules.Module):
     # --== Tree management ==--
 
 
-    def getDirContent(self, directory):
+    def __startLoading(self, tree, row):
+        """ Tell the user that the contents of row is being loaded """
+        name = tree.getItem(row, ROW_NAME)
+        # TODO Put this style somewhere, we're using it at multiple places
+        tree.setItem(row, ROW_NAME, '%s  <span size="smaller" foreground="#909090">%s</span>' % (name, _('loading...')))
+
+
+    def __stopLoading(self, tree, row):
+        """ Tell the user that the contents of row has been loaded"""
+        name  = tree.getItem(row, ROW_NAME)
+        index = name.find('<')
+
+        if index != -1:
+            tree.setItem(row, ROW_NAME, name[:index-2])
+
+
+    def getDirContents(self, directory):
         """ Return a tuple of sorted rows (directories, playlists, mediaFiles) for the given directory """
         playlists   = []
         mediaFiles  = []
@@ -151,12 +169,12 @@ class FileExplorer(modules.Module):
 
         for (file, path) in tools.listDir(directory, self.showHiddenFiles):
             if isdir(path):
-                directories.append((consts.icoDir, unicode(file, errors='replace'), TYPE_DIR, path))
+                directories.append((consts.icoDir, cgi.escape(unicode(file, errors='replace')), TYPE_DIR, path))
             elif isfile(path):
                 if media.isSupported(file):
-                    mediaFiles.append((consts.icoMediaFile, unicode(file, errors='replace'), TYPE_FILE, path))
+                    mediaFiles.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_FILE, path))
                 elif playlist.isSupported(file):
-                    playlists.append((consts.icoMediaFile, unicode(file, errors='replace'), TYPE_PLIST, path))
+                    playlists.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_PLIST, path))
 
         playlists.sort(key=self.sortKey)
         mediaFiles.sort(key=self.sortKey)
@@ -165,21 +183,21 @@ class FileExplorer(modules.Module):
         return (directories, playlists, mediaFiles)
 
 
-    def exploreDir(self, tree, parent, directory):
-        """ List the content of the given directory and append it to the tree as a child of parent """
-
-        # Make sure the durectory is still there
-        if not os.path.exists(directory):
-            return
-
-        directories, playlists, mediaFiles = self.getDirContent(directory)
+    def exploreDir(self, tree, parent, directory, fakeChild=None):
+        """
+            List the contents of the given directory and append it to the tree as a child of parent
+            If fakeChild is not None, remove it when the real contents has been loaded
+        """
+        directories, playlists, mediaFiles = self.getDirContents(directory)
 
         tree.appendRows(directories, parent)
         tree.appendRows(playlists,   parent)
         tree.appendRows(mediaFiles,  parent)
 
-        if directories != []:
-            idle_add(self.updateChildren(tree, parent).next)
+        if fakeChild is not None:
+            tree.removeRow(fakeChild)
+
+        idle_add(self.updateChildren(tree, parent).next)
 
 
     def updateChildren(self, tree, parent):
@@ -187,7 +205,7 @@ class FileExplorer(modules.Module):
         for child in tree.iterChildren(parent):
             # Only directories need to be updated and since they are always on top, we can stop if we find something else
             if tree.getItem(child, ROW_TYPE) != TYPE_DIR:
-                yield False
+                break
 
             # Make sure it's readable
             directory = tree.getItem(child, ROW_FULLPATH)
@@ -214,6 +232,10 @@ class FileExplorer(modules.Module):
 
             yield True
 
+
+        if parent is not None:
+            self.__stopLoading(tree, parent)
+
         yield False
 
 
@@ -222,7 +244,7 @@ class FileExplorer(modules.Module):
         if treePath is None: directory = self.folders[self.currFolder]
         else:                directory = tree.getItem(treePath, ROW_FULLPATH)
 
-        directories, playlists, mediaFiles = self.getDirContent(directory)
+        directories, playlists, mediaFiles = self.getDirContents(directory)
 
         disk       = directories + playlists + mediaFiles
         diskIndex  = 0
@@ -293,15 +315,21 @@ class FileExplorer(modules.Module):
 
     def onKeyPressed(self, tree, event):
         """ A key has been pressed """
-        if gtk.gdk.keyval_name(event.keyval) == 'F5':
-            self.refresh(self.tree)
+        keyname = gtk.gdk.keyval_name(event.keyval)
+
+        if keyname == 'F5':       self.refresh(tree)
+        elif keyname == 'plus':   tree.expandRows()
+        elif keyname == 'Left':   tree.collapseRows()
+        elif keyname == 'Right':  tree.expandRows()
+        elif keyname == 'minus':  tree.collapseRows()
+        elif keyname == 'space':  tree.switchRows()
+        elif keyname == 'Return': self.playSelection(tree, True)
 
 
     def onRowExpanded(self, tree, path):
         """ Replace the fake child by the real children """
-        fakeChild = tree.getChild(path, 0)
-        self.exploreDir(tree, path, tree.getItem(path, ROW_FULLPATH))
-        tree.removeRow(fakeChild)
+        idle_add(self.__startLoading, tree, path)
+        idle_add(self.exploreDir, tree, path, tree.getItem(path, ROW_FULLPATH), tree.getChild(path, 0))
 
 
     def onRowCollapsed(self, tree, path):
