@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,10 +16,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import dbus, gui, modules, traceback
+import dbus, gui, media.track, modules, traceback
 
 from tools     import consts, prefs
-from media     import track
 from gettext   import gettext as _
 from tools.log import logger
 
@@ -191,10 +190,10 @@ class IMStatus(modules.Module):
 
     def init(self):
         """ Initialize the module """
+        self.track     = None   # Current track
         self.status    = ''     # The currently used status
         self.paused    = False  # True if the current track is paused
         self.clients   = []     # Clients currently active
-        self.trackInfo = None   # Information about the current track
         self.cfgWindow = None   # Configuration window
 
         # Detect active clients
@@ -214,9 +213,9 @@ class IMStatus(modules.Module):
             logger.error('[%s] Error while initializing\n\n%s' % (MOD_NAME, traceback.format_exc()))
 
 
-    def __format(self, string, trackInfo):
+    def __format(self, string, track):
         """ Replace the special fields in the given string by their corresponding value and sanitize the result """
-        result = trackInfo.format(string)
+        result = track.format(string)
 
         if len(prefs.get(__name__, 'sanitized-words', DEFAULT_SANITIZED_WORDS)) != 0:
             lowerResult = result.lower()
@@ -237,18 +236,18 @@ class IMStatus(modules.Module):
                 client[IM_INSTANCE].setStatusMsg(account, status)
 
 
-    def onNewTrack(self, trackInfo):
+    def onNewTrack(self, track):
         """ A new track is being played """
-        self.status    = self.__format(prefs.get(__name__, 'status-msg', DEFAULT_STATUS_MSG), trackInfo)
-        self.paused    = False
-        self.trackInfo = trackInfo
+        self.track  = track
+        self.status = self.__format(prefs.get(__name__, 'status-msg', DEFAULT_STATUS_MSG), track)
+        self.paused = False
         self.setStatusMsg(self.status)
 
 
     def onStopped(self):
         """ The current track has been stopped """
-        self.paused    = False
-        self.trackInfo = None
+        self.track  = None
+        self.paused = False
         if prefs.get(__name__, 'stop-action', DEFAULT_STOP_ACTION) == STOP_SET_STATUS:
             self.setStatusMsg(prefs.get(__name__, 'stop-status', DEFAULT_STOP_STATUS))
 
@@ -257,7 +256,7 @@ class IMStatus(modules.Module):
         """ The current track has been paused """
         self.paused = True
         if prefs.get(__name__, 'update-on-paused', DEFAULT_UPDATE_ON_PAUSED):
-            self.setStatusMsg(self.status + ' ' + _('[paused]'))
+            self.setStatusMsg(_('%(status)s [paused]') % {'status': self.status})
 
 
     def onUnpaused(self):
@@ -328,9 +327,9 @@ class IMStatus(modules.Module):
             prefs.set(__name__, 'stop-status', self.cfgWindow.getWidget('txt-stopStatus').get_text())
         self.cfgWindow.hide()
         # Update status
-        if self.trackInfo is not None:
-            self.status = self.__format(prefs.get(__name__, 'status-msg', DEFAULT_STATUS_MSG), self.trackInfo)
-            if self.paused: self.setStatusMsg(self.status + ' ' + _('[paused]'))
+        if self.track is not None:
+            self.status = self.__format(prefs.get(__name__, 'status-msg', DEFAULT_STATUS_MSG), self.track)
+            if self.paused: self.setStatusMsg(_('%(status)s [paused]') % {'status': self.status})
             else:           self.setStatusMsg(self.status)
 
 
@@ -344,7 +343,7 @@ class IMStatus(modules.Module):
         helpDlg.addSection(_('Customizing the Status'),
                            _('You can set the status to any text you want. Before setting it, the module replaces all fields of '
                              'the form {field} by their corresponding value. Available fields are:')
-                           + '\n\n' + track.getFormatSpecialFields(False))
+                           + '\n\n' + media.track.getFormatSpecialFields(False))
         helpDlg.addSection(_('Markup'),
                            _('You can use the Pango markup language to format the text. More information on that language is '
                              'available on the following web page:')
