@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,20 +16,24 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, collections, gtk, gui, media, modules, os, shutil, tools, urllib
+import cgi, collections, gtk, gui, media, modules, os, shutil, sys, tools
 
-from gui             import fileChooser, help, questionMsgBox, extTreeview, extListview, progressDlg, selectPath
-from tools           import consts, prefs, pickleLoad, pickleSave
-from gettext         import gettext as _
-from os.path         import isdir, isfile
-from gobject         import idle_add, TYPE_STRING, TYPE_INT, TYPE_PYOBJECT
-from gui.progressDlg import ProgressDlg
+from gui                   import fileChooser, help, questionMsgBox, extTreeview, extListview, progressDlg, selectPath
+from tools                 import consts, prefs, pickleLoad, pickleSave
+from gettext               import ngettext, gettext as _
+from os.path               import isdir, isfile
+from gobject               import idle_add, TYPE_STRING, TYPE_INT, TYPE_PYOBJECT
+from tools.log             import logger
+from gui.progressDlg       import ProgressDlg
+from media.track.fileTrack import FileTrack
 
 MOD_INFO = ('Library', _('Library'), _('Organize your music by tags instead of files'), [], False, True)
 MOD_L10N = MOD_INFO[modules.MODINFO_L10N]
+MOD_NAME = MOD_INFO[modules.MODINFO_NAME]
 
 
 # Constants
+VERSION                  = 2                                      # Used to enforce compatibility
 ROOT_PATH                = os.path.join(consts.dirCfg, 'Library') # Path where libraries are stored
 PREFS_DEFAULT_PREFIXES   = {'the ': None}                         # Prefixes are put at the end of artists' names
 PREFS_DEFAULT_LIBRARIES  = {}                                     # No libraries at first
@@ -111,7 +115,7 @@ class Library(modules.Module):
 
         self.tree.get_column(0).set_cell_data_func(txtRdr,    self.__drawCell)
         self.tree.get_column(0).set_cell_data_func(pixbufRdr, self.__drawCell)
-        self.tree.setIsDraggableFunc(self.__isDraggable)
+        self.tree.setIsDraggableFunc(self.isDraggable)
         self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
         # Scroll window
         self.scrolled = gtk.ScrolledWindow()
@@ -126,18 +130,31 @@ class Library(modules.Module):
         self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
 
 
+    def isDraggable(self):
+        """ Only playable rows can be dragged """
+        for row in self.tree.iterSelectedRows():
+            if row[ROW_TYPE] != TYPE_NONE:
+                return True
+        return False
+
+
     def __drawCell(self, column, cell, model, iter):
         """ Use a different background color for alphabetical headers """
         if model.get_value(iter, ROW_TYPE) == TYPE_HEADER: cell.set_property('cell-background-gdk', self.tree.style.bg[gtk.STATE_PRELIGHT])
         else:                                              cell.set_property('cell-background',     None)
 
 
-    def __isDraggable(self):
-        """ Return True if the selection is draggable """
-        for row in self.tree.iterSelectedRows():
-            if row[ROW_TYPE] == TYPE_HEADER:
-                return False
-        return True
+    def __createEmptyLibrary(self, name):
+        """ Create bootstrap files for a new library """
+        # Make sure that the root directory of all libraries exists
+        if not isdir(ROOT_PATH):
+            os.mkdir(ROOT_PATH)
+        # Start from an empty library
+        libPath = os.path.join(ROOT_PATH, name)
+        if isdir(libPath):
+            shutil.rmtree(libPath)
+        os.mkdir(libPath)
+        pickleSave(os.path.join(libPath, 'files'), {})
 
 
     def refreshLibrary(self, parent, libName, path, creation=False):
@@ -149,12 +166,17 @@ class Library(modules.Module):
         progress = ProgressDlg(parent, header, _('The directory is scanned for media files. This can take some time.\nPlease wait.'))
         yield True
 
-        db         = {}                                         # The dictionnary used to create the library
-        queue      = collections.deque((path,))                 # Faster structure for appending/removing elements
-        libPath    = os.path.join(ROOT_PATH, libName)           # Location of the library
-        mediaFiles = []                                         # All media files found
-        newLibrary = {}                                         # Reflect the current file structure of the library
-        oldLibrary = pickleLoad(os.path.join(libPath, 'files')) # Previous file structure of the same library
+        libPath = os.path.join(ROOT_PATH, libName)   # Location of the library
+
+        # If the version number has changed or does not exist, don't reuse any existing file and start from scratch
+        if not os.path.exists(os.path.join(libPath, 'VERSION_%u' % VERSION)):
+            self.__createEmptyLibrary(libName)
+
+        db         = {}                                                                # The dictionnary used to create the library
+        queue      = collections.deque((path,))                                        # Faster structure for appending/removing elements
+        mediaFiles = []                                                                # All media files found
+        newLibrary = {}                                                                # Reflect the current file structure of the library
+        oldLibrary = pickleLoad(os.path.join(libPath, 'files'))                        # Previous file structure of the same library
 
         while len(queue) != 0:
             currDir      = queue.pop()
@@ -174,21 +196,22 @@ class Library(modules.Module):
                         directories.append(fullPath)
                     elif isfile(fullPath) and media.isSupported(filename):
                         if filename in oldFiles: files[filename] = oldFiles[filename]
-                        else:                    files[filename] = [-1, [0, '', '', '', 0, fullPath]]
+                        else:                    files[filename] = [-1, FileTrack(fullPath)]
 
             # Determine which files need to be updated
-            for filename, (oldMTime, tags) in files.iteritems():
-                mTime = os.stat(tags[media.NFO_FIL]).st_mtime
+            for filename, (oldMTime, track) in files.iteritems():
+                mTime = os.stat(track.getFilePath()).st_mtime
                 if mTime != oldMTime:
-                    files[filename] = [mTime, media.readInfo((tags[media.NFO_FIL],))[0]]
+                    files[filename] = [mTime, media.getTrackFromFile(track.getFilePath())]
 
             newLibrary[currDir] = (currDirMTime, directories, files)
-            mediaFiles.extend([file for mTime, file in files.itervalues()])
+            mediaFiles.extend([track for mTime, track in files.itervalues()])
             queue.extend(directories)
 
             # Update the progress dialog
             try:
-                progress.pulse(_('Scanning directories (%u tracks found)' % len(mediaFiles)))
+                text = ngettext('Scanning directories (one track found)', 'Scanning directories (%(nbtracks)u tracks found)', len(mediaFiles))
+                progress.pulse(text % {'nbtracks': len(mediaFiles)})
                 yield True
             except progressDlg.CancelledException:
                 progress.destroy()
@@ -204,10 +227,10 @@ class Library(modules.Module):
 
         # Create the database
         for track in mediaFiles:
-            album, artist = track[media.NFO_ALB], track[media.NFO_ART]
+            album = track.getExtendedAlbum()
 
-            if len(track) > 6 and track[media.NFO_AAR] != consts.UNKNOWN_ALBUM_ARTIST:
-                artist = track[media.NFO_AAR]
+            if track.hasAlbumArtist(): artist = track.getAlbumArtist()
+            else:                      artist = track.getArtist()
 
             if artist in db:
                 allAlbums = db[artist]
@@ -236,6 +259,9 @@ class Library(modules.Module):
             shutil.rmtree(libPath)
             os.mkdir(libPath)
 
+        # Put a version number
+        tools.touch(os.path.join(libPath, 'VERSION_%u' % VERSION))
+
         overallNbAlbums  = 0
         overallNbTracks  = 0
         overallNbArtists = len(db)
@@ -252,11 +278,11 @@ class Library(modules.Module):
 
             albums = []
             for index, (name, tracks) in enumerate(db[artist].iteritems()):
-                length           = sum([track[media.NFO_LEN] for track in tracks])
+                length           = sum([track.getLength() for track in tracks])
                 overallNbTracks += len(tracks)
 
                 albums.append((name, str(index), len(tracks), length))
-                pickleSave(os.path.join(artistPath, str(index)), sorted(tracks, key = lambda track: track[media.NFO_NUM]))
+                pickleSave(os.path.join(artistPath, str(index)), sorted(tracks, key = lambda track: track.getNumber()))
 
             albums.sort(key = lambda album: album[0].lower())
             pickleSave(os.path.join(artistPath, 'albums'), albums)
@@ -279,9 +305,10 @@ class Library(modules.Module):
 
 
     def __getTracksFromSelectedRows(self, tree):
-        """ Return a list of tracks with all the associated tags, based on selected rows """
+        """ Return a list of tracks based on selected rows """
         tracks = []
-        for row in tree.getSelectedRows():
+        for currPath in tree.getSelectedPaths():
+            row = tree.getRow(currPath)
             if row[ROW_TYPE] == TYPE_TRACK:
                 tracks.append(row[ROW_TAGS])
             elif row[ROW_TYPE] == TYPE_ALBUM:
@@ -289,9 +316,17 @@ class Library(modules.Module):
             elif row[ROW_TYPE] == TYPE_ARTIST:
                 for album in pickleLoad(os.path.join(row[ROW_FULLPATH], 'albums')):
                     tracks.extend(pickleLoad(os.path.join(row[ROW_FULLPATH], album[ALB_INDEX])))
+            elif row[ROW_TYPE] == TYPE_HEADER:
+                for path in xrange(currPath[0]+1, sys.maxint):
+                    if not tree.isValidPath(path):
+                        break
 
-        for track in tracks:
-            track[media.NFO_FIL] = 'file://' + track[media.NFO_FIL]
+                    row = tree.getRow(path)
+                    if row[ROW_TYPE] == TYPE_HEADER:
+                        break
+
+                    for album in pickleLoad(os.path.join(row[ROW_FULLPATH], 'albums')):
+                        tracks.extend(pickleLoad(os.path.join(row[ROW_FULLPATH], album[ALB_INDEX])))
 
         return tracks
 
@@ -300,8 +335,8 @@ class Library(modules.Module):
         """ Replace/extend the tracklist """
         tracks = self.__getTracksFromSelectedRows(tree)
 
-        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET_EXT, {'files': tracks, 'playNow': True})
-        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD_EXT, {'files': tracks})
+        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
+        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
     def showPopupMenu(self, tree, button, time, path):
@@ -315,16 +350,10 @@ class Library(modules.Module):
             self.popup.get_widget('item-add').connect('activate',      lambda widget: self.playSelection(tree, False))
             self.popup.get_widget('item-refresh').connect('activate',  lambda widget: idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next))
 
-        # Determine whether at least one selected item is playable
-        playable = False
-        for row in self.tree.iterSelectedRows():
-            if row[ROW_TYPE] != TYPE_HEADER:
-                playable = True
-                break
-
-        # Show the popup menu
+        playable = path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE
         self.popup.get_widget('item-add').set_sensitive(playable)
         self.popup.get_widget('item-play').set_sensitive(playable)
+
         self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
 
 
@@ -334,9 +363,18 @@ class Library(modules.Module):
         path     = os.path.join(ROOT_PATH, name)
         prevChar = None
 
-        # Create the rows, with alphabetical header is needed
+        # Make sure the version number is the good one
+        if not os.path.exists(os.path.join(path, 'VERSION_%u' % VERSION)):
+            logger.error('[%s] Version number does not match, loading of library "%s" aborted' % (MOD_NAME, name))
+            error = _('This library is deprecated, please refresh it.')
+            tree.replaceContent([(consts.icoError, error, TYPE_NONE, None, None)])
+            return
+
+        # Create the rows, with alphabetical header if needed
         for artist in pickleLoad(os.path.join(path, 'artists')):
-            currChar = artist[ART_NAME][0]
+            if len(artist[ART_NAME]) != 0: currChar = unicode(artist[ART_NAME], errors='replace')[0]
+            else:                          currChar = prevChar
+
             if prevChar is None or (prevChar != currChar and not (prevChar.isdigit() and currChar.isdigit())):
                 prevChar = currChar
                 if currChar.isdigit(): rows.append((None, '<b>0 - 9</b>',         TYPE_HEADER, None, None))
@@ -369,7 +407,7 @@ class Library(modules.Module):
     def loadTracks(self, tree, node, fakeChild):
         """ Initial load of all tracks of the given node, assuming it is of type TYPE_ALBUM """
         allTracks = pickleLoad(tree.getItem(node, ROW_FULLPATH))
-        rows      = [(consts.icoMediaFile, '%02u. %s' % (track[media.NFO_NUM], cgi.escape(track[media.NFO_TIT])), TYPE_TRACK, track[media.NFO_FIL], track) for track in allTracks]
+        rows      = [(consts.icoMediaFile, '%02u. %s' % (track.getNumber(), cgi.escape(track.getTitle())), TYPE_TRACK, track.getFilePath(), track) for track in allTracks]
 
         tree.appendRows(rows, node)
         tree.removeRow(fakeChild)
@@ -390,7 +428,7 @@ class Library(modules.Module):
         """ A mouse button has been pressed """
         if event.button == 3:
             self.showPopupMenu(tree, event.button, event.time, path)
-        elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path is not None:
+        elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE:
             if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playSelection(tree, True)
             elif tree.row_expanded(path):                         tree.collapse_row(path)
             else:                                                 tree.expand_row(path, False)
@@ -398,16 +436,21 @@ class Library(modules.Module):
 
     def onKeyPressed(self, tree, event):
         """ A key has been pressed """
-        if gtk.gdk.keyval_name(event.keyval) == 'F5':
-            idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next)
+        keyname = gtk.gdk.keyval_name(event.keyval)
+
+        if keyname == 'F5':       idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next)
+        elif keyname == 'plus':   tree.expandRows()
+        elif keyname == 'Left':   tree.collapseRows()
+        elif keyname == 'Right':  tree.expandRows()
+        elif keyname == 'minus':  tree.collapseRows()
+        elif keyname == 'space':  tree.switchRows()
+        elif keyname == 'Return': self.playSelection(tree, True)
 
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        tracks    = self.__getTracksFromSelectedRows(tree)
-        allFields = '\n'.join([' '.join([urllib.pathname2url(str(field)) for field in track]) for track in tracks])
-
-        selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, allFields)
+        serializedTracks = '\n'.join([track.serialize() for track in self.__getTracksFromSelectedRows(tree)])
+        selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, serializedTracks)
 
 
     def addAllExplorers(self):
@@ -497,16 +540,6 @@ class Library(modules.Module):
 
         if result is not None:
             name, path = result
-            # Make sure that the root directory of all libraries exists
-            if not isdir(ROOT_PATH):
-                os.mkdir(ROOT_PATH)
-            # Start from an empty library
-            libPath = os.path.join(ROOT_PATH, name)
-            if isdir(libPath):
-                shutil.rmtree(libPath)
-            os.mkdir(libPath)
-            pickleSave(os.path.join(libPath, 'files'), {})
-            # And 'refresh' it
             idle_add(self.refreshLibrary(self.cfgWindow, name, path, True).next)
 
 
