@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ class StatusIcon(modules.Module):
         """ The current track has been paused """
         self.isPaused = True
         self.statusIcon.set_from_pixbuf(self.icoPause)
-        self.statusIcon.set_tooltip(self.tooltip + ' ' + _('[paused]'))
+        self.statusIcon.set_tooltip(_('%(tooltip)s [paused]') % {'tooltip': self.tooltip})
 
 
     def onUnpause(self):
