@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,12 +20,17 @@ import gtk, gui, media, modules, os.path, tools, urllib
 
 from tools           import consts
 from gettext         import gettext as _
-from gobject         import TYPE_STRING, TYPE_INT
+from gobject         import TYPE_STRING, TYPE_INT, TYPE_PYOBJECT
 from media.track     import Track
 from gui.extListview import ExtListView
 
 MOD_INFO = ('Tracklist', 'Tracklist', '', [], True, False)
 
+# Default preferences
+PREFS_DEFAULT_REPEAT_STATUS = False
+
+
+TRACKLIST_NB_FIELDS = 7
 
 (
     ROW_ICO,   # Icon drawn in front of the row
@@ -34,8 +39,8 @@ MOD_INFO = ('Tracklist', 'Tracklist', '', [], True, False)
     ROW_ART,   # Track artist
     ROW_ALB,   # Track album
     ROW_LEN,   # Track length in seconds
-    ROW_PTH    # Full path to the file
-) = range(7)
+    ROW_TRK    # The Track object
+) = range(TRACKLIST_NB_FIELDS)
 
 
 class Tracklist(modules.Module):
@@ -46,8 +51,8 @@ class Tracklist(modules.Module):
         modules.Module.__init__(self, (consts.MSG_CMD_TRACKLIST_SET,     consts.MSG_CMD_TRACKLIST_CLR,     consts.MSG_CMD_TRACKLIST_ADD,
                                        consts.MSG_CMD_TOGGLE_PAUSE,      consts.MSG_CMD_NEXT,              consts.MSG_CMD_PREVIOUS,
                                        consts.MSG_EVT_NEED_BUFFER,       consts.MSG_EVT_STOPPED,           consts.MSG_EVT_PAUSED,
-                                       consts.MSG_EVT_UNPAUSED,          consts.MSG_EVT_TRACK_ENDED,       consts.MSG_CMD_TRACKLIST_SET_EXT,
-                                       consts.MSG_CMD_TRACKLIST_ADD_EXT, consts.MSG_CMD_TRACKLIST_SHUFFLE, consts.MSG_EVT_APP_STARTED))
+                                       consts.MSG_EVT_UNPAUSED,          consts.MSG_EVT_TRACK_ENDED_OK,    consts.MSG_EVT_TRACK_ENDED_ERROR,
+                                       consts.MSG_CMD_TRACKLIST_SHUFFLE, consts.MSG_EVT_APP_STARTED))
 
 
     def onAppStarted(self):
@@ -74,7 +79,7 @@ class Tracklist(modules.Module):
                    (_('Artist'), [(txtLRdr, TYPE_STRING)],                           (ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), True),
                    (_('Album'),  [(txtLRdr, TYPE_STRING)],                           (ROW_ALB, ROW_NUM, ROW_TIT),          True),
                    (_('Length'), [(txtRRdr, TYPE_INT)],                              (ROW_LEN,),                           False),
-                   (None,        [(None, TYPE_STRING)],                              (None,),                              False))
+                   (None,        [(None, TYPE_PYOBJECT)],                            (None,),                              False))
 
         self.list = ExtListView(columns, True, consts.DND_TARGETS.values())
         self.list.get_column(4).set_cell_data_func(txtRRdr, self.fmtLength)
@@ -89,6 +94,8 @@ class Tracklist(modules.Module):
         self.btnClear.connect('clicked',   lambda widget: modules.postMsg(consts.MSG_CMD_TRACKLIST_CLR))
         self.btnRepeat.connect('toggled',  self.onButtonRepeat)
         self.btnShuffle.connect('clicked', lambda widget: modules.postMsg(consts.MSG_CMD_TRACKLIST_SHUFFLE))
+        # Restore preferences
+        self.btnRepeat.set_active(tools.prefs.get(__name__, 'repeat-status', PREFS_DEFAULT_REPEAT_STATUS))
         # Try to associate an icon to the shuffle button
         if consts.fileImgShuffle is not None: wTree.get_widget('img-shuffle').set_from_file(consts.fileImgShuffle)
         else:                                 wTree.get_widget('img-shuffle').hide()
@@ -100,8 +107,8 @@ class Tracklist(modules.Module):
     # Helper functions
     def hasNext(self):                      return self.list.hasMarkUnder(len(self.list)-1)
     def hasPrev(self):                      return self.list.hasMarkAbove(0)
-    def getAllFiles(self):                  return [row[ROW_PTH] for row in self.list.getAllRows()]
-    def getAllTracks(self):                 return [Track(row[1:], i, len(self.list)) for i, row in enumerate(self.list.getAllRows())]
+    def getAllFiles(self):                  return [row[ROW_TRK].getFilePath() for row in self.list.iterAllRows()]
+    def getAllTracks(self):                 return [row[ROW_TRK] for row in self.list.iterAllRows()]
     def fmtLength(self, col, cll, mdl, it): cll.set_property('text', tools.sec2str(mdl.get_value(it, ROW_LEN)))
 
 
@@ -112,77 +119,81 @@ class Tracklist(modules.Module):
         self.btnShuffle.set_sensitive(sensitive)
 
 
-    def jumpTo(self, trackIndex):
+    def jumpTo(self, trackIdx):
         """ Jump to the track located at the given index """
-        if self.list.hasMark():
+        if self.list.hasMark() and self.list.getItem(self.list.getMark(), ROW_ICO) != consts.icoError:
             self.list.setItem(self.list.getMark(), ROW_ICO, consts.icoNull)
-        self.list.setMark(trackIndex)
-        self.list.scroll_to_cell(trackIndex)
-        self.list.setItem(trackIndex, ROW_ICO, consts.icoPlay)
-        modules.postMsg(consts.MSG_CMD_PLAY,        {'filename': self.list.getItem(trackIndex, ROW_PTH)})
-        modules.postMsg(consts.MSG_EVT_NEW_TRACK,   {'track': Track(self.list.getRow(trackIndex)[1:], trackIndex+1, len(self.list))})
+        self.list.setMark(trackIdx)
+        self.list.scroll_to_cell(trackIdx)
+        self.list.setItem(trackIdx, ROW_ICO, consts.icoPlay)
+        modules.postMsg(consts.MSG_CMD_PLAY,        {'uri': self.list.getItem(trackIdx, ROW_TRK).getURI()})
+        modules.postMsg(consts.MSG_EVT_NEW_TRACK,   {'track': self.list.getRow(trackIdx)[ROW_TRK]})
         modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {'hasPrevious': self.hasPrev(), 'hasNext': self.hasNext() or self.btnRepeat.get_active()})
 
 
-    def onTrackEnded(self):
+    def onTrackEnded(self, withError):
         """ The current track has ended, jump to the next one if any """
-        if   self.hasNext():              self.jumpTo(self.list.getMark()+1)
-        elif self.btnRepeat.get_active(): self.jumpTo(0)
-        else:                             modules.postMsg(consts.MSG_CMD_STOP)
+        currIdx = self.list.getMark()
+
+        # If an error occurred with the current track, flag it as such
+        if withError:
+            self.list.setItem(currIdx, ROW_ICO, consts.icoError)
+
+        # Find the next 'playable' track (not already flagged)
+        if self.btnRepeat.get_active(): nbTracks = len(self.list)
+        else:                           nbTracks = len(self.list) - 1 - currIdx
+
+        for i in xrange(nbTracks):
+            currIdx = (currIdx + 1) % len(self.list)
+
+            if self.list.getItem(currIdx, ROW_ICO) != consts.icoError:
+                self.jumpTo(currIdx)
+                return
+
+        modules.postMsg(consts.MSG_CMD_STOP)
 
 
     def onBufferingNeeded(self):
         """ The current track is close to its end, so we try to buffer the next one to avoid gaps """
-        if self.hasNext():                modules.postMsg(consts.MSG_CMD_BUFFER, {'filename': self.list.getItem(self.list.getMark()+1, ROW_PTH)})
-        elif self.btnRepeat.get_active(): modules.postMsg(consts.MSG_CMD_BUFFER, {'filename': self.list.getItem(0, ROW_PTH)})
+        if self.hasNext():
+            modules.postMsg(consts.MSG_CMD_BUFFER, {'uri': self.list.getItem(self.list.getMark()+1, ROW_TRK).getURI()})
+        elif self.btnRepeat.get_active():
+            modules.postMsg(consts.MSG_CMD_BUFFER, {'uri': self.list.getItem(0, ROW_TRK).getURI()})
 
 
-    def insert(self, tracks, readTags=True, position=None):
-        """
-            Insert some tracks in the tracklist, append them if position is None
-            Refer to set() for the meaning of readTags
-        """
-        if not readTags:
-            tracks = [[consts.icoNull] + track[0:6] for track in tracks]                    # Remove "album artist", we don't use it
-        else:
-            files  = [track[7:] for track in tracks if track.startswith('file://')]
-            tracks = [[consts.icoNull] + track[0:6] for track in media.getTracks(files)]    # Remove "album artist", we don't use it
+    def insert(self, tracks, position=None):
+        """ Insert some tracks in the tracklist, append them if position is None """
+        rows = [[consts.icoNull, track.getNumber(), track.getTitle(), track.getArtist(), track.getExtendedAlbum(), track.getLength(), track]
+                    for track in tracks]
 
-            for track in tracks:
-                track[ROW_PTH] = 'file://' + track[ROW_PTH]
+        for row in rows:
+            self.playtime += row[ROW_LEN]
 
-        if len(tracks) != 0:
-            for track in tracks:
-                self.playtime += track[ROW_LEN]
-
-            self.__updateButtons(True)
-            self.list.insertRows(tracks, position)
+        self.__updateButtons(True)
+        self.list.insertRows(rows, position)
 
 
-    def set(self, tracks, playNow, readTags=True):
-        """
-            Replace the tracklist, clear it if tracks is None
-            If readTags is True, tracks is just a list of filenames and the tags must be read from them
-            If readTags is False, tracks already contains all tags information (e.g., as returned by media.readInfo())
-        """
+    def set(self, tracks, playNow):
+        """ Replace the tracklist, clear it if tracks is None """
         self.playtime = 0
         self.list.clear()
 
-        if tracks is not None:
-            self.insert(tracks, readTags)
-
-        if len(self.list) == 0:
+        if tracks is None:
             self.__updateButtons(False)
             modules.postMsg(consts.MSG_CMD_STOP)
             modules.postMsg(consts.MSG_EVT_NEW_TRACKLIST, {'tracks': [], 'playtime': 0})
-        elif playNow:
-            self.jumpTo(0)
+        else:
+            self.insert(tracks)
+            if playNow and len(self.list) != 0:
+                self.jumpTo(0)
 
 
     def onStopped(self):
         """ Playback has been stopped """
         if self.list.hasMark():
-            self.list.setItem(self.list.getMark(), ROW_ICO, consts.icoNull)
+            currTrack = self.list.getMark()
+            if self.list.getItem(currTrack, ROW_ICO) != consts.icoError:
+                self.list.setItem(currTrack, ROW_ICO, consts.icoNull)
             self.list.clearMark()
 
 
@@ -194,14 +205,11 @@ class Tracklist(modules.Module):
 
     def savePlaylist(self):
         """ Save the current tracklist to a playlist """
-        destDir = os.path.dirname(self.list.getItem(0, ROW_PTH))
-
-        if destDir.startswith('file://'): outFile = gui.fileChooser.save(self.window, _('Save playlist'), 'playlist.m3u', destDir[7:])
-        else:                             outFile = gui.fileChooser.save(self.window, _('Save playlist'), 'playlist.m3u')
+        destDir = os.path.dirname(self.list.getItem(0, ROW_TRK).getFilePath())
+        outFile = gui.fileChooser.save(self.window, _('Save playlist'), 'playlist.m3u', destDir)
 
         if outFile is not None:
-            allFiles = [file[7:] for file in self.getAllFiles() if file.startswith('file://')]
-            media.playlist.save(allFiles, outFile)
+            media.playlist.save(self.getAllFiles(), outFile)
 
 
     def cropSelection(self):
@@ -214,7 +222,7 @@ class Tracklist(modules.Module):
         """ Remove all the currently selected tracks """
         for row in self.list.iterSelectedRows():
             self.playtime -= row[ROW_LEN]
-            if row[ROW_ICO] != consts.icoNull:
+            if row[ROW_ICO] == consts.icoPlay:
                 modules.postMsg(consts.MSG_CMD_STOP)
 
         self.list.removeSelectedRows()
@@ -254,13 +262,12 @@ class Tracklist(modules.Module):
         elif msg == consts.MSG_EVT_STOPPED:                                  self.onStopped()
         elif msg == consts.MSG_EVT_UNPAUSED:                                 self.onPausedToggled(consts.icoPlay)
         elif msg == consts.MSG_EVT_APP_STARTED:                              self.onAppStarted()
-        elif msg == consts.MSG_EVT_TRACK_ENDED:                              self.onTrackEnded()
+        elif msg == consts.MSG_EVT_TRACK_ENDED_OK:                           self.onTrackEnded(False)
+        elif msg == consts.MSG_EVT_TRACK_ENDED_ERROR:                        self.onTrackEnded(True)
         elif msg == consts.MSG_EVT_NEED_BUFFER:                              self.onBufferingNeeded()
         elif msg == consts.MSG_CMD_TRACKLIST_CLR:                            self.set(None, False)
-        elif msg == consts.MSG_CMD_TRACKLIST_SET:                            self.set(params['files'], params['playNow'])
-        elif msg == consts.MSG_CMD_TRACKLIST_ADD:                            self.insert(params['files'])
-        elif msg == consts.MSG_CMD_TRACKLIST_ADD_EXT:                        self.insert(params['files'], False)
-        elif msg == consts.MSG_CMD_TRACKLIST_SET_EXT:                        self.set(params['files'], params['playNow'], False)
+        elif msg == consts.MSG_CMD_TRACKLIST_SET:                            self.set(params['tracks'], params['playNow'])
+        elif msg == consts.MSG_CMD_TRACKLIST_ADD:                            self.insert(params['tracks'])
         elif msg == consts.MSG_CMD_TRACKLIST_SHUFFLE:                        self.shuffleTracklist()
         elif msg == consts.MSG_CMD_PREVIOUS and self.hasPrev():              self.jumpTo(self.list.getMark()-1)
         elif msg == consts.MSG_CMD_TOGGLE_PAUSE and not self.list.hasMark(): self.jumpTo(0)
@@ -274,6 +281,7 @@ class Tracklist(modules.Module):
 
     def onButtonRepeat(self, btn):
         """ The 'repeat' button has been pressed """
+        tools.prefs.set(__name__, 'repeat-status', self.btnRepeat.get_active())
         if self.list.hasMark():
             modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {'hasPrevious': self.hasPrev(), 'hasNext': self.hasNext() or self.btnRepeat.get_active()})
 
@@ -295,6 +303,12 @@ class Tracklist(modules.Module):
     def onListModified(self, list):
         """ Some rows have been added/removed/moved """
         self.__updateButtons(len(list) != 0)
+
+        # Update playlist length and playlist position for all tracks
+        for position, row in enumerate(self.list.getAllRows()):
+            row[ROW_TRK].setPlaylistPos(position + 1)
+            row[ROW_TRK].setPlaylistLen(len(self.list))
+
         modules.postMsg(consts.MSG_EVT_NEW_TRACKLIST, {'tracks': self.getAllTracks(), 'playtime': self.playtime})
         modules.postMsg(consts.MSG_EVT_TRACK_MOVED,   {'hasPrevious': self.hasPrev(), 'hasNext':  self.hasNext()})
 
@@ -309,18 +323,12 @@ class Tracklist(modules.Module):
 
         # A list of filenames
         if dndId == consts.DND_DAP_URI or dndId == consts.DND_URI:
-            tracks   = [urllib.url2pathname(uri) for uri in dragData.data.split()]
-            readTags = True
-        # A list of tracks with all tags provided
+            tracks = media.getTracks([urllib.url2pathname(uri) for uri in dragData.data.split()])
+        # A list of tracks
         elif dndId == consts.DND_DAP_TRACKS:
-            tracks   = [[urllib.url2pathname(field) for field in track.split()] for track in dragData.data.split('\n')]
-            readTags = False
+            tracks = [media.track.unserialize(serialTrack) for serialTrack in dragData.data.split('\n')]
 
-            for track in tracks:
-                track[media.NFO_NUM] = int(track[media.NFO_NUM])
-                track[media.NFO_LEN] = int(track[media.NFO_LEN])
-
-        if dropInfo is None: self.insert(tracks, readTags)
-        else:                self.insert(tracks, readTags, dropInfo[0][0])
+        if dropInfo is None: self.insert(tracks)
+        else:                self.insert(tracks, dropInfo[0][0])
 
         context.finish(True, False, time)
