@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,15 +22,21 @@ import consts, cPickle, gtk.glade, os
 __dirCache = {}
 
 def listDir(directory, listHiddenFiles=False):
-    """ Return a list of tuples (filename, path) with the given directory content """
-    # The dircache module sorts the list of files, and either it's not needed or it's not sorted the way we want
+    """
+        Return a list of tuples (filename, path) with the given directory content
+        The dircache module sorts the list of files, and either it's not needed or it's not sorted the way we want
+    """
     if directory in __dirCache: cachedMTime, list = __dirCache[directory]
     else:                       cachedMTime, list = None, None
 
-    mTime = os.stat(directory).st_mtime
+    if os.path.exists(directory): mTime = os.stat(directory).st_mtime
+    else:                         mTime = 0
 
     if mTime != cachedMTime:
-        list                  = os.listdir(directory)
+        # Make sure it's readable
+        if os.access(directory, os.R_OK | os.X_OK): list = os.listdir(directory)
+        else:                                       list = []
+
         __dirCache[directory] = (mTime, list)
 
     return [(filename, os.path.join(directory, filename)) for filename in list if listHiddenFiles or filename[0] != '.']
@@ -64,3 +70,19 @@ def pickleSave(file, data):
     output = open(file, 'w')
     cPickle.dump(data, output)
     output.close()
+
+
+def touch(filePath):
+    """ Equivalent to the Linux 'touch' command """
+    os.system('touch "%s"' % filePath)
+
+
+def percentEncode(string):
+    """
+        Percent-encode all the bytes in the given string
+        I didn't find a Python method to do that
+    """
+    mask  = '%%%X' * len(string)
+    bytes = tuple([ord(c) for c in string])
+
+    return mask % bytes
