@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ socketTimeout = 10
 
 # --- Strings
 appName      = 'Decibel Audio Player'
-appVersion   = '0.10'
+appVersion   = '0.11'
 appNameShort = 'decibel-audio-player'
 
 
@@ -97,12 +97,16 @@ dbusInterface = 'org.silentblade.decibel.remoteinterface'
 
 
 # --- Tracks
-UNKNOWN_TITLE        = _('Unknown title')
+UNKNOWN_DATE         = 0
+UNKNOWN_GENRE        = _('Unknown Genre')
+UNKNOWN_TITLE        = _('Unknown Title')
 UNKNOWN_ALBUM        = _('Unknown Album')
 UNKNOWN_ARTIST       = _('Unknown Artist')
 UNKNOWN_LENGTH       = 0
-UNKNOWN_TRACKNUMBER  = 0
-UNKNOWN_ALBUM_ARTIST = ''
+UNKNOWN_MB_TRACKID   = 0
+UNKNOWN_DISC_NUMBER  = 0
+UNKNOWN_TRACK_NUMBER = 0
+UNKNOWN_ALBUM_ARTIST = _('Unknown Album Artist')
 
 
 # --- Stock icons
@@ -111,6 +115,7 @@ icoDir       = tmpLbl.render_icon(gtk.STOCK_DIRECTORY,   gtk.ICON_SIZE_MENU)
 icoPlay      = tmpLbl.render_icon(gtk.STOCK_MEDIA_PLAY,  gtk.ICON_SIZE_MENU)
 icoPause     = tmpLbl.render_icon(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU)
 icoCdrom     = tmpLbl.render_icon(gtk.STOCK_CDROM,       gtk.ICON_SIZE_MENU)
+icoError     = tmpLbl.render_icon(gtk.STOCK_CANCEL,      gtk.ICON_SIZE_MENU)
 icoMediaDir  = tmpLbl.render_icon(gtk.STOCK_DIRECTORY,   gtk.ICON_SIZE_MENU).copy()  # We need a copy to modify it
 icoMediaFile = tmpLbl.render_icon(gtk.STOCK_FILE,        gtk.ICON_SIZE_MENU).copy()  # We need a copy to modify it
 
@@ -144,7 +149,7 @@ DND_TARGETS = {
     # --== COMMANDS ==--
 
     # GStreamer player
-    MSG_CMD_PLAY,             # Play a file         Parameters: 'filename'
+    MSG_CMD_PLAY,             # Play a resource     Parameters: 'uri'
     MSG_CMD_STOP,             # Stop playing        Parameters:
     MSG_CMD_SEEK,             # Jump to a position  Parameters: 'seconds'
     MSG_CMD_SET_VOLUME,       # Change the volume   Parameters: 'value'
@@ -154,28 +159,29 @@ DND_TARGETS = {
     # Tracklist
     MSG_CMD_NEXT,              # Play the next track       Parameters:
     MSG_CMD_PREVIOUS,          # Play the previous track   Parameters:
-    MSG_CMD_TRACKLIST_SET,     # Replace tracklist         Parameters: 'files', 'playNow'
-    MSG_CMD_TRACKLIST_ADD,     # Extend tracklist          Parameters: 'files'
+    MSG_CMD_TRACKLIST_SET,     # Replace tracklist         Parameters: 'tracks', 'playNow'
+    MSG_CMD_TRACKLIST_ADD,     # Extend tracklist          Parameters: 'tracks'
     MSG_CMD_TRACKLIST_CLR,     # Clear tracklist           Parameters:
     MSG_CMD_TRACKLIST_SHUFFLE, # Shuffle the tracklist     Parameters:
 
-    MSG_CMD_TRACKLIST_SET_EXT, # Tags are provided with the filenames (as returned by media.readInfo())   Parameters: 'files', 'playNow'
-    MSG_CMD_TRACKLIST_ADD_EXT, # Same as above                                                            Parameters: 'files'
-
     # Explorers
-    MSG_CMD_EXPLORER_ADD,     # Add a new explorer    Parameters: 'modName', 'expName', 'icon', 'widget'
-    MSG_CMD_EXPLORER_REMOVE,  # Remove an explorer    Parameters: 'modName', 'expName'
+    MSG_CMD_EXPLORER_ADD,      # Add a new explorer    Parameters: 'modName', 'expName', 'icon', 'widget'
+    MSG_CMD_EXPLORER_REMOVE,   # Remove an explorer    Parameters: 'modName', 'expName'
+
+    # Covers
+    MSG_CMD_SET_COVER,         # Cover file for the given track     Parameters: 'track', 'path'
 
     # --== EVENTS ==--
 
     # Current track
-    MSG_EVT_PAUSED,           # Paused                              Parameters:
-    MSG_EVT_STOPPED,          # Stopped                             Parameters:
-    MSG_EVT_UNPAUSED,         # Unpaused                            Parameters:
-    MSG_EVT_NEW_TRACK,        # The current track has changed       Parameters: 'track'
-    MSG_EVT_NEED_BUFFER,      # The next track should be buffered   Parameters:
-    MSG_EVT_TRACK_ENDED,      # The current track has ended         Parameters:
-    MSG_EVT_TRACK_POSITION,   # New position in the current track   Parameters: 'seconds'
+    MSG_EVT_PAUSED,              # Paused                                             Parameters:
+    MSG_EVT_STOPPED,             # Stopped                                            Parameters:
+    MSG_EVT_UNPAUSED,            # Unpaused                                           Parameters:
+    MSG_EVT_NEW_TRACK,           # The current track has changed                      Parameters: 'track'
+    MSG_EVT_NEED_BUFFER,         # The next track should be buffered                  Parameters:
+    MSG_EVT_TRACK_POSITION,      # New position in the current track                  Parameters: 'seconds'
+    MSG_EVT_TRACK_ENDED_OK,      # The current track has ended                        Parameters:
+    MSG_EVT_TRACK_ENDED_ERROR,   # The current track has ended because of an error    Parameters:
 
     # GStreamer player
     MSG_EVT_VOLUME_CHANGED,   # The volume has changed   Parameters: 'value'
